import {Component, OnInit, ViewChild} from '@angular/core';
import { Router } from '@angular/router';

import { Utils } from '../core/utils/utils.service';
import {Patient, PatientService} from "../core/service/patient.service";
import {NzModalComponent} from "ng-zorro-antd/modal";
import {NzMessageService} from "ng-zorro-antd/message";
import {FormBuilder, FormGroup, Validators} from "@angular/forms";

@Component({
    selector: 'app-patient-list',
    templateUrl: './patient-list.component.html',
    styleUrls: ['./patient-list.component.css']
})
export class PatientListComponent implements OnInit {

    validatePatientForm!: FormGroup;
    patientModalObj: Patient;
    @ViewChild('patientModal') patientModal: NzModalComponent;
    patientModalIsEdit: boolean = false;
    patientUpdating: boolean = false;
    isShowPatientModal: boolean = false;
    patientList: any[] = [];
    oldPatientIdentifier: string = '';

    constructor(private utils: Utils,
                private router: Router,
                private patientService: PatientService,
                private _message: NzMessageService,
                private fb: FormBuilder) {

    }

    async ngOnInit() {
        this.loadList();
    }

    async loadList() {
        this.patientList = await this.patientService.getAllPatient().toPromise() as any;
        this.formatPatientList();
    }

    formatPatientList() {
        let date = new Date();
        for (let item of this.patientList) {
            item.age = this.utils.getAgeByDateStr(item.birthdate, date.toString());
        }
    }

    clickPatientDataReady(event : any, patient: any) {
        event.cancelBubble = true;
        this.router.navigate(['patient/data/ready'], { queryParams: { patientIdentifier: patient.patientIdentifier } });
    }

    async onOpenPatientModelBtn(event? : any, model? : Patient) {
        if (event) {
            event.cancelBubble = true;
        }
        this.patientModalIsEdit = !!model;
        if (model) {
            this.oldPatientIdentifier = model.patientIdentifier;
        }
        this.patientModalObj = model ? {...model} : new Patient();
        this.validatePatientForm = this.fb.group({
            patientIdentifier:[this.patientModalObj.patientIdentifier,[Validators.required]],
            admissionId:[this.patientModalObj.admissionId,[Validators.required]],
            patientName:[this.patientModalObj.patientName,[Validators.required]],
            gender: [this.patientModalObj.gender],
            birthdate: [this.patientModalObj.birthdate],
            dept: [this.patientModalObj.dept],
            patientStatus: [this.patientModalObj.patientStatus],
        });
        this.isShowPatientModal = true;
        this.patientModal.open();
    }

    async createPatient() {
        let patient = this.validatePatientForm.value;
        patient.meetingId = 1;
        patient.creatorId = sessionStorage.getItem("user_id");
        patient.creatorName = sessionStorage.getItem("user_name");
        patient.patientStatus = '数据准备';
        let result = await this.patientService.addPatient(patient).toPromise();
        if (result) {
            await this.addAppDesignAndCdssEntity(patient);
        }
        this._message.success('创建成功');
        this.patientModal.close();
        this.loadList();
    }

    async updatePatient() {
        let patient = this.validatePatientForm.value;
        patient.patientId = this.patientModalObj.patientId;
        patient.meetingId = this.patientModalObj.meetingId;
        let result = await this.patientService.updatePatient(patient).toPromise();
        if (result && this.oldPatientIdentifier != patient.patientIdentifier) {
            await this.updateAppDesignAndCdssEntity(patient);
        }
        this._message.success('更新成功');
        this.patientModal.close();
        this.loadList();
    }


    async deletePatient(patient: Patient) {
        let result = await this.patientService.deletePatient(patient.patientId).toPromise();
        if (result) {
            await this.deleteAppDesignAndCdssEntity(patient);
        }
        this._message.success('删除成功');
        this.loadList();
    }

    closePatientModal() {
        this.isShowPatientModal = false;
        this.patientModal.close();
    }

    async clickPatientCard(patient: Patient) {
        this.router.navigate(['patient/view'], { queryParams: { patientId: patient.patientId , patientIdentifier: patient.patientIdentifier } });
    }

    async addAppDesignAndCdssEntity(patient: any) {
        let entity : any = {
            systemId: "TumorBoard",
            entityTypeId: 1,
            dataJson: JSON.stringify({"patient_id" : patient.patientIdentifier}),
            source: 0,
            entityId: patient.patientIdentifier,
            creatorId : sessionStorage.getItem("user_id") ? sessionStorage.getItem("user_id") : 1,
            creatorName : sessionStorage.getItem("user_name") ? sessionStorage.getItem("user_name") : "unknow",
        };
        let results: any[] = await Promise.all([
            this.patientService.addAppDesignEntity(entity).toPromise() as any,
            this.patientService.addCdssEntity(entity).toPromise() as any,
        ]);
    }

    async updateAppDesignAndCdssEntity(patient: any) {
        let results: any[] = await Promise.all([
            this.patientService.updateAppDesignEntity(this.oldPatientIdentifier, patient.patientIdentifier).toPromise() as any,
            this.patientService.updateCdssEntity(this.oldPatientIdentifier, patient.patientIdentifier).toPromise() as any,
        ]);
    }

    async deleteAppDesignAndCdssEntity(patient: any) {
        let results: any[] = await Promise.all([
            this.patientService.deleteAppDesignEntity(patient.patientIdentifier).toPromise() as any,
            this.patientService.deleteCdssEntity(patient.patientIdentifier).toPromise() as any,
        ]);
    }

    async syncPatientInfo() {
        let patient = this.validatePatientForm.value;
        if (patient.patientIdentifier) {
            let result = await this.patientService.getCdrPatientBasicInfo(patient.patientIdentifier).toPromise() as any;
            this.validatePatientForm = this.fb.group({
                patientIdentifier:[patient.patientIdentifier,[Validators.required]],
                admissionId:[result.ENCOUNTERIDENTIFIER,[Validators.required]],
                patientName:[result.PATIENTNAME,[Validators.required]],
                gender: [result.GENDER_NAME],
                birthdate: [new Date(result.BIRTHDATE)],
                dept: [result.DEPT]
            });
        } else {
            this._message.error("尚未输入病人标识符");
        }
    }
}
